package com.dzeio.openhealth.ui.weight

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.adapters.WeightAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentListWeightBinding
import com.dzeio.openhealth.ui.home.HomeViewModel
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collectLatest

@AndroidEntryPoint
class ListWeightFragment :
    BaseFragment<HomeViewModel, FragmentListWeightBinding>(HomeViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentListWeightBinding =
        FragmentListWeightBinding::inflate

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = WeightAdapter()
        adapter.onItemClick = {
            findNavController().navigate(
                ListWeightFragmentDirections.actionNavListWeightToNavEditWeight(
                    it.id
                )
            )
            //EditWeightDialog().show(requireActivity().supportFragmentManager, "dialog")
        }
        recycler.adapter = adapter

        viewLifecycleOwner.lifecycleScope.launchWhenCreated {
            viewModel.fetchWeights().collectLatest {
                val itt = it.toMutableList()
                itt.sortWith { o1, o2 -> if (o1.timestamp > o2.timestamp) -1 else 1 }
                adapter.set(itt)
            }
        }


    }
}